public with sharing class TestLayout {
    
    public enum Visibility {SENDER, ACCOUNT}
    
    
    public void testLayout(){
    	//create a layout
    	String pkgId = createDocumentExtraction();
    	ESignLiveAPIObjects.package_x createLayout = createLayout(pkgId,'Sample_Document_Extraction','Layout Created from Apex SDK', TestLayout.Visibility.ACCOUNT);
    	String layoutId = createLayout.id;
    	
    	
    	//retrieve all layouts
    	integer totalCount = getLayoutCount();
    	integer index = 0;
    	
    	System.debug('Total layout count: ' + totalCount);
    	while(index < totalCount){
	    	List<ESignLiveAPIObjects.package_x> layouts = getLayouts(index,index+49);
	    	for(ESignLiveAPIObjects.package_x layout: layouts){
	    		System.debug('Layout name: ' + layout.name + ' with id: ' + layout.id);
	    	}
	    	index += 50;
    	}
    	
    	
    	//apply layout
    	applyLayout('QAcCjW98liytMV7tocfM6j78BP8=', 'bbafd99a30946447e3f214c075b5a2bed642b43deb795b68', layoutId);
    	
    	
    	//delete layout
    	//ESignLiveSDK sdk = new ESignLiveSDK();
    	//sdk.deletePackage(layoutId);
    }
    
    
    public String createDocumentExtraction(){
    	ESignLiveSDK sdk = new ESignLiveSDK();
         
        //Create package
        ESignLiveAPIObjects.Package_x pkg = new ESignLiveAPIObjects.Package_x();
        pkg.name = 'Test Document Extraction - ' + Datetime.now().format();
        pkg.status = ESignLiveAPIObjects.PackageStatus.DRAFT;
        
        //Create Roles
        String roleId1 = 'Signer1';
        ESignLiveAPIObjects.Role role1 = new ESignLiveAPIObjects.Role();
        role1.signers = sdk.createRolesSigner('sigenr1_firstname', 'signer1_lastname', 'signer1@mailinator.com', 'CEO', 'ABC Bank');
        role1.id = roleId1;
        role1.name = roleId1;
        
        String roleId2 = 'Preparer1';
        ESignLiveAPIObjects.Role role2 = new ESignLiveAPIObjects.Role();
        role2.signers = sdk.createRolesSigner('sigenr2_firstname', 'signer2_lastname', 'signer2@mailinator.com', 'Applicant', 'ABC Company');
        role2.id = roleId2;
        role2.name = roleId2;
        
        pkg.roles = new List<ESignLiveAPIObjects.Role>{role1,role2};    //add role

        
        //Prepare Documents Blob
        String document1Name = 'Sample_Document_Extraction';
        StaticResource sr = [SELECT Id, Body FROM StaticResource WHERE Name = 'test_document_extraction' LIMIT 1];
        Map<String,Blob> documentBlobMap = new Map<String,Blob>();
        documentBlobMap.put(document1Name, sr.Body);
         
        //Create Document Metadata
        ESignLiveAPIObjects.Document document1 = new ESignLiveAPIObjects.Document();
        document1.name = document1Name;
        document1.id = document1Name;
        document1.extract = true;
        
        pkg.documents = new List<ESignLiveAPIObjects.Document>{document1};    //add document
        
    	//Send package One Step
        String packageId = sdk.createPackage(pkg,documentBlobMap);
        System.debug('PackageId: ' + packageId);
        return packageId;
    }
    
    
    public ESignLiveAPIObjects.package_x createLayout(string packageId, string documentId, string layoutName, TestLayout.Visibility visibility){
    	String resourceUrl = '/layouts';
    	String requestPayload = '{"name":"'+layoutName+'","type":"LAYOUT","id":"'+packageId+'","visibility":"'+visibility.name()+'","documents":[{"id":"'+documentId+'"}]}';
        HttpResponse response = doPost(resourceUrl, requestPayload);
		
        // Check response
        if(response.getStatusCode() == 200)
        {
            try
            {
                return (ESignLiveAPIObjects.package_x)JSON.deserialize(response.getBody(), ESignLiveAPIObjects.package_x.class);
            }
            catch(Exception e)
            {
                throw new ESignLiveRESTAPIHelper.ESignLiveRestAPIHelperException('Error create Layout ' + e.getMessage());
            }
        }   
        else
        {
            throw new ESignLiveRESTAPIHelper.ESignLiveRestAPIHelperException('Error create Layout' + response.getStatusCode() + ' - ' + response.getStatus() + ' - ' +  response.getBody());
        }
    }
    
    
    private HttpResponse doPost(String resourceUrl, String requestBody)
    {
        e_SignLive_Connection_Settings__c connectionSettings = e_SignLive_Connection_Settings__c.getInstance('Main');    
        //Cleanup requestBody string before sending it out
        String outboundRequestBody = ESignLiveJSONHelper.prepareOutboundJSON(requestBody);
        System.debug('=> outboundRequestBody:'+outboundRequestBody);

        // Build Request
        HttpRequest request = new HttpRequest();

        request.setEndpoint(connectionSettings.Endpoint__c + resourceUrl);
        request.setBody(outboundRequestBody);
        request.setMethod('POST');
        request.setHeader('Authorization', 'Basic ' + connectionSettings.API_Key__c);
        request.setHeader('Content-type', 'application/json');
        request.setHeader('Accept', 'application/json');

        // Send Request
        HttpResponse response = (new Http()).send(request);

        return response;
    }
    
    
    
    public integer getLayoutCount(){
    	// Send Request
        String resourceUrl = '/layouts';
        HttpResponse response = doGet(resourceUrl);
		
        // Check response
        if(response.getStatusCode() == 200)
        {
            try
            {
            	Map<String,Object> responseObject = (Map<String,Object>)JSON.deserializeUntyped(response.getBody());
            	return (integer)responseObject.get('count');
            }
            catch(Exception e)
            {
                throw new ESignLiveRESTAPIHelper.ESignLiveRestAPIHelperException('Error converting templates json ' + e.getMessage());
            }
        }   
        else
        {
            throw new ESignLiveRESTAPIHelper.ESignLiveRestAPIHelperException('Error get templates ' + response.getStatusCode() + ' - ' + response.getStatus() + ' - ' +  response.getBody());
        }
    }
    
     private HttpResponse doGet(String resourceUrl){
        e_SignLive_Connection_Settings__c connectionSettings = e_SignLive_Connection_Settings__c.getInstance('Main');    
         // Build Request
        HttpRequest request = new HttpRequest();
        request.setEndpoint(connectionSettings.Endpoint__c + resourceUrl);
        request.setMethod('GET');
        request.setHeader('Authorization', 'Basic ' + connectionSettings.API_Key__c);
        request.setHeader('Content-type', 'application/json');

        // Send Request
        HttpResponse response = (new Http()).send(request);

        if(response != null && (response.getHeader('Content-Type')).contains('json'))
        {
            System.debug('=> Formating for JSON response');
            // Format responseBody string before returning
            String inboundResponseBody = ESignLiveJSONHelper.prepareInboundJSON(response.getBody());
            response.setBody(inboundResponseBody);
        }

        return response;
    }
    
     public List<ESignLiveAPIObjects.package_x> getLayouts(integer from_x, integer to){
    	// Send Request
        String resourceUrl = '/layouts?from=' + from_x + '&to=' + to;
        HttpResponse response = doGetWithoutPrepareInboundJSON(resourceUrl);
		
        // Check response
        if(response.getStatusCode() == 200)
        {
            try
            {
            	Map<String,Object> responseObject = (Map<String,Object>)JSON.deserializeUntyped(response.getBody());
            	List<Object> layoutObjects = (List<Object>)responseObject.get('results');
            	List<ESignLiveAPIObjects.package_x> layouts = new List<ESignLiveAPIObjects.package_x>();
            	for(Object layout : layoutObjects){
            		layouts.add((ESignLiveAPIObjects.package_x)JSON.deserialize(JSON.serialize(layout),ESignLiveAPIObjects.package_x.class));
            	}
            	return layouts;
            }
            catch(Exception e)
            {
                throw new ESignLiveRESTAPIHelper.ESignLiveRestAPIHelperException('Error converting layouts json ' + e.getMessage());
            }
        }   
        else
        {
            throw new ESignLiveRESTAPIHelper.ESignLiveRestAPIHelperException('Error get layouts: '+resourceUrl+' ' + response.getStatusCode() + ' - ' + response.getStatus() + ' - ' +  response.getBody());
        }
    }
    
    
    
    private HttpResponse doGetWithoutPrepareInboundJSON(String resourceUrl){
        e_SignLive_Connection_Settings__c connectionSettings = e_SignLive_Connection_Settings__c.getInstance('Main');    
         // Build Request
        HttpRequest request = new HttpRequest();
        request.setEndpoint(connectionSettings.Endpoint__c + resourceUrl);
        request.setMethod('GET');
        request.setHeader('Authorization', 'Basic ' + connectionSettings.API_Key__c);
        request.setHeader('Content-type', 'application/json');

        // Send Request
        HttpResponse response = (new Http()).send(request);

        return response;
    }
    
    
    
     public void applyLayout(String packageId, String documentId, String layoutId){
    	String resourceUrl = '/packages/'+packageId+'/documents/'+documentId+'/layout?layoutId='+layoutId;
        HttpResponse response = doPost(resourceUrl, '');
		
        // Check response
        if(response.getStatusCode() != 200)
        {
            throw new ESignLiveRESTAPIHelper.ESignLiveRestAPIHelperException('Error apply Layout' + response.getStatusCode() + ' - ' + response.getStatus() + ' - ' +  response.getBody());
        }
    }
    
    
    
    
}